# Automatic Content Updates

A github action is running every 20 minutes. Checking for new content and creating a PR with the changes if necessary.

### Technical Details

* A mimo user is used for logging in (mimo-bot@getmimo.com)
* The file `.package-version` contains the version of the last publish.
* If a PR already exists but is not merged. The action will not create a new PR

# Lesson Format Specification

## Lesson Structure

### Basic structure

Every lesson is a `XML` file at its core.
Originally the format was meant to be pure `HTML`, but after some time we realized that this was too limiting.
Because of this, we switched to XML, but keep the HTML header, `header` and `body` tag for legacy reasons.

This is the base structure that every lesson must contain:

```html
<!DOCTYPE html>
<html>
<body>
  <!-- INSERT BODY HERE -->
</body>
</html>
```

### Modules

Every lesson consists of one or more "modules".
A module is defined by its opening and closing tags.

E.g. `<p>` & `</p>` defines a text module.

The following example is the simplest possible lesson:

```html
<!DOCTYPE html>
<html>
<body>
    <p>
        Text
    </p>
</body>
</html>
```

* Text
* Code
  * Spell
  * Fill in the gap
  * Selection
  * Ordering
  * Validated Input
  * Output
* Multiple Choice
* Ordering
* Image
* WebView
* [Files](https://github.com/getmimo/Lessons/blob/master/format_lesson_files.md)
* [Instructions](https://github.com/getmimo/Lessons/blob/master/instructions.md)

### Text

```html
<p>
    This is text.
    <br />
    Line breaks are specified manually.
    &nbsp;&nbsp;This line starts after two whitespaces
</p>
```

### Code

The code module is used for displaying code and specifies interactions that the user can do, like selections, ordering and fill in the gap.

Originally the code module looked very similar to the text module.
After some time, we noticed a bunch of limitations of this format, mainly the whitespace handling and extensibility for new interactions.

For further documentation, go to the [Code Module Specification](format_codemodule_v2.md)

### Images

Image modules are specified with the `img` tag.

The image ID is specified in the `src` tag. The tutorial metadata structure contains a list of images for the lessons, so they can be prefetched by the app, before the lessons are opened.

```html
<img src="98f64586-58c5-455d-bf6f-6035e03c0927" />
```

### WebView Module

The WebView module can contain arbitrary HTML content that is displayed in a WebView

```html
<webview>
    <![CDATA[
    <p>Real HTML!</p>
    ]]>
</webview>
```

**NOTE**: The CDATA element *must* be included, otherwise the HTML inside the WebView isn't detected properly

### Multiple-choice

The `<selection multiple="multiple">` tag must contain at least two `<item>` tags:

```html
<selection multiple id="2">
  <item correct="true"><number>10</number></item>
  <item correct="true"><number>20</number></item>
  <item correct="false"><number>30</number></item>
</selection multiple>
```


### Ordering

The `<ordering isCode="true">` tag must contain at least two `<item>` tags:

```html
<ordering isCode="true">
  <item>10</item>
  <item>20</item>
  <item>30</item>
</ordering>
```

If the multiple choice options contain code instead of text, `selection` element is marked with `isCode="true"`

### Visible-if

The `visible-if` attribute marks a module as hidden until the user triggers an event.
E.g. The user clicks the "Run" or "Solve" button.

```html
<p visible-if="correct">
  Text is only visible if user selects all answers correctly.
</p>
<p visible-if="wrong">
  Text is only visible if user selects at least one answer incorrectly.
</p>
```
