# Code Module v2

## Goals

* easier extensible -> future additions should force as little code changes as possible
* more flexible -> by tokenizing the text it is easier to reference specific parts. This should make it easy to add new interactive variations

## Format

``` swift
[1,2,3].map {
    print($0)
}
```

```xml
<code version="2" name="index.js">
    <content>
        <text value="[" />
        <text value="1" class="keyword" />
        <text value="," />
        <text value="2" class="keyword" />
        <text value="," />
        <text value="3" class="keyword" />
        <text value="]." />
        <text value="map" class="keyword" />
        <text value=" {" />
        <text type="break" />
        <text value="    print($" />
        <text value="0" class="keyword" />
        <text value=")" />
        <text type="break" />
        <text value="}" />
    </content>
</code>
```

### Interactive Variations (**Input**)

The variations are specified separately and reference the tokens which they relate to.

* **Ordering** - Order the lines
* **Selection** - Swap text with a selection element
* **Fill The Gap**
* **Spell** - Swap text with a spell element
* **Type** - Swap text with a type element

#### Ordering

The ordering interaction does not need any more information. The client will group tokens only separated by `break` text and use them as rows for the ordering module.

* **single** type attribute

``` xml
<code>
    <content>
        <text></text>
    </content>
    <interactions>
        <interaction type="orderTheLines" />
    </interactions>
</code>
```

#### Selection

The Author selects a text block and has to specify alternative (incorrect) options.

* **startIndex** && **endIndex** attribute
* `options` array
* each `option` has a `content`

``` xml
    <code>
        <content>
            <text value="test print test"></text>
        </content>
        <interactions>
            <interaction type="selection" startIndex="5" endIndex="9">
                <options>
                    <option correct="false">
                        <content>
                            <text value="write" class="keyword" />
                        </content>
                    </option>
                    <option correct="false">
                        <content>
                            <text value="System.out" class="keyword" />
                        </content>
                    </option>
                </options>
            </interaction>
        </interactions>
    </code>
```

#### Fill The Gap

The Author selects text blocks. Additionally, the Author is able to provide wrong alternatives for the selected text blocks. The selected text blocks and the wrong alternatives are displayed for the user, who has to insert them into the correct position of the codeblock.

``` xml
    <code>
        <content>
            <text></text>
        </content>
        <interactions>
            <interaction type="fillTheGap">
                <options>
                    <option startIndex="0" endIndex="6"/>
                    <option startIndex="9" endIndex="13"/>
                    <option startIndex="14" endIndex="20"/>
                    <option correct="false">
                        <content>
                            <text value="option" />
                        </content>
                    </option>
                </options>
            </interaction>
        </interactions>
    </code>
```

#### Spell

Select multiple a blocks which get split into single chars on the client. The user has to select the correct order of the chars to form the word.

* only a **single** option of the `options` array is shown to the users. Select randomly on the client

``` xml
<modules>
    <code>
        <content>
            <text></text>
        </content>
        <interactions>
            <interaction type="spell" startIndex="0" endIndex="6">
            </interaction>
        </interactions>
    <code/>
</modules>
```

#### Validated Input

Select a text block which gets split into single chars on the client. The user has to select the correct order of the chars to form the word.

* only a **single** option of the `options` array is shown to the users. Select randomly on the client

``` xml
<modules>
    <code>
        <content>
            <text></text>
        </content>
        <interactions>
            <interaction type="validatedInput">
                <options>
                    <option startIndex="0" endIndex="6"/>
                    <option startIndex="9" endIndex="13"/>
                    <option startIndex="14" endIndex="20"/>
                </options>
            </interaction>
        </interactions>
    <code/>
</modules>
```

### Code **Output**

In general a `output` object is specified for the module directly.

``` xml
<modules>
    <code>
        <content>
            <text></text>
        </content>
        <interactions>
            <interaction />
        </interactions>
        <output>
            <content>
                <text value="N1, you are the queen  👑" />
            </content>
        </output>
    </code>
</modules>
```

The author can overwrite the output for each interactive variation if necessary. If the variation contains its own outputs the client has to use these.

* **not** used right now

``` xml
<modules>
    <code>
        <content>
            <text></text>
        </content>
        <interactions>
            <interaction type="orderTheLines">
                <outputs>
                    <output>
                        <content>
                            <text value="N1, you are the queen 👑" />
                        </content>
                    </output>
                </outputs>
            </interaction>
        </interactions>
        <output>
            <content>
                <text value="N1, you are the king  👑" />
            </content>
        </output>
    </code>
</modules>
```

### Collapse/expand code lines
For longer code blocks, irrelevant code should be hidden by default, and the user should be able to expand it, and collapse it back.
In the `<code version="2">` module the optional element `<collapsibleLines>` specifies which lines of code should be collapsed.
Line numbering starts from `0`. `startLineIndex` and `endLineIndex` are inclusive.
Collapsible lines will not be used along with the `Ordering` interaction, because right now we cannot choose only certain lines to be ordered.

``` xml
<modules>
    <code version="2">
        <content>
            <text value="some initial irrelevant lines of code">
            <text type="break" />
            <text value="some initial irrelevant lines of code">
            <text type="break" />
            <text value="some initial irrelevant lines of code">
            <text type="break" />
            <text value="list = [" />
            <text value="3" class="number" />
            <text value="," />
            <text value="7" class="number" />
            <text value="," />
            <text value="1" class="number" />
            <text value="," />
            <text value="4" class="number" />
            <text value="," />
            <text value="9" class="number" />
            <text value="," />
            <text value="2" class="number" />
            <text value="," />
            <text value="5" class="number" />
            <text value="]" />
            <text type="break" />
            <text value="length = len(list)" />
            <text type="break" />
            <text value="for" class="keyword" />
            <text value=" i " />
            <text value="in" class="keyword" />
            <text value=" range(length):" />
            <text type="break" />
            <text value=" " />
            <text value="for" class="keyword" />
            <text value=" j " />
            <text value="in" class="keyword" />
            <text value=" range(length" />
            <text value="-1" class="number" />
            <text value="):" />
            <text type="break" />
            <text value="  " />
            <text value="if" class="keyword" />
            <text value=" list[j] :" />
            <text type="break" />
            <text value="some other irrelevant line of code">
            <text type="break" />
        </content>
        <interactions>
            <interaction type="fillTheGap" isDefault="true">
                <options><option startIndex="98" endIndex="102" />
                    <option startIndex="102" endIndex="103" />
                    <option startIndex="103" endIndex="104" />
                    <option startIndex="104" endIndex="105" />
                </options>
            </interaction>
        </interactions>
        <collapsibleLines>
          <collapseAt startLineIndex="0" endLineIndex="2"/>
          <collapseAt startLineIndex="8" endLineIndex="8"/>
        </collapsibleLines>
    </code>
</modules>
```
