const fs = require('fs-extra');

function deleteDirectoryRecursively(path) {
  if (fs.existsSync(path)) {
    fs.readdirSync(path).forEach(function(file, index) {
      var curPath = path + '/' + file;
      if (fs.lstatSync(curPath).isDirectory()) {
        // recurse
        deleteDirectoryRecursively(curPath);
      } else {
        // delete file
        fs.unlinkSync(curPath);
      }
    });
    fs.rmdirSync(path);
  }
}

async function replaceOldTrackDirectories(__parentDirName) {
  // Replace old tracks directory with latest tracks directory
  deleteDirectoryRecursively(__parentDirName + '/tracks');
  try {
    await fs.rename(
      __parentDirName + '/TempTracks/tracks',
      __parentDirName + '/tracks',
    );
    console.log('Tracks directory replaced');

    // Replace old icons directory with latest icons directory
    deleteDirectoryRecursively(__parentDirName + '/icons');
    await fs.rename(
      __parentDirName + '/TempTracks/icons',
      __parentDirName + '/icons',
    );
    console.log('Icons directory replaced');

    // Replace old icon xcassets directory with latest icon xcassets directory
    deleteDirectoryRecursively(__parentDirName + '/icons.xcassets');
    await fs.rename(
      __parentDirName + '/TempTracks/icons.xcassets',
      __parentDirName + '/icons.xcassets',
    );
    console.log('Icons xcassets directory replaced');
    
    // Replace old glossary directory with latest glossary directory
    deleteDirectoryRecursively(__parentDirName + '/glossary');
    await fs.rename(
      __parentDirName + '/TempTracks/glossary',
      __parentDirName + '/glossary',
    );
    console.log('glossary directory replaced');
    
    // Replace old featured.json with latest featured.json
    await fs.rename(
      __parentDirName + '/TempTracks/featured.json',
      __parentDirName + '/featured.json',
    );
    console.log('featured.json replaced');

    // Replace old explore.json with latest explore.json
    // Disabled until we have the explore.json in our backend system
    // await fs.rename(
    //   __parentDirName + '/TempTracks/explore.json',
    //   __parentDirName + '/explore.json',
    // );
    // console.log('Explore JSON replaced');
  } catch (err) {
    console.error(err);
  }
}

function oauthBody() {
  return JSON.stringify({
    client_id: process.env.OAUTH_CLIENT_ID,
    connection: 'Alexandria-Users',
    device: 'Web',
    grant_type: 'password',
    password: process.env.ACCOUNT_PASSWORD,
    scope: 'roles openid offline_access email profile',
    username: process.env.ACCOUNT_EMAIL,
  });
}

function alexandriaHeaders(bearerToken) {
  console.log(bearerToken);
  return {
    Authorization: bearerToken,
    accept: 'application/json',
    'Content-Type': 'application/json-patch+json',
  };
}

function updateContentVersionNumber(newVersionNumber) {
  fs.writeFileSync('./.package-version', newVersionNumber);
}

module.exports = {
  replaceOldTrackDirectories,
  oauthBody,
  alexandriaHeaders,
  updateContentVersionNumber,
};
